﻿/* 
    Hi! Welcome the source of the Game Ports Mod!

    If you're a fan of the mod looking to see how it was made, or you just like the code, give my accounts a follow!
    Twitter: https://twitter.com/bryceedev
    Youtube: https://youtube.com/c/SpooksHD
    Roblox: https://www.roblox.com/users/51429227/profile <- Because I'm cool like that, also play my games! :)

    If you see anything that can be optimised or done better please message me on any of those platforms or even on my steam
    account - I really like optimising what needs to be (but couldn't because this is just a hobby project).

    If you have any ideas and end up adding any or just have some PLEASE shoot them my way!

    S.N. - I comment things that I believe need to be explained logically or need to be referenced to later,
    so for the most part everything should be easy to understand. If not, please contact me for any questions!

    - Bryce
*/

{
    // Get our path
    const OurDirectory = GDT.getRelativePath()

    // Add research item
    Research.GamePorts = {
        id: "GamePorts",
        name: "Game Porting".localize(),
        pointsCost: 50,
        duration: 13E3,
        cost: 10E4,
        devCost: 4E4,
        canResearch: function (company) {
            return company.isLaterOrEqualThan(8, 6);
        },
        category: "Game Design",
        categoryDisplayName: "Game Design",
        complete: function () {
            if(GameManager.company) {
                GameManager.company.notifications.push(new Notification("Thinking with Ports!".localize("heading"), "Welcome to the world of Porting! This brand new technology will make it so you can take any game you've previously made and port it to a new platform! {n} You'll see a new button in the menu just for Porting Games, so when you're deciding to go make that next big hit; save on the new ideas, {0} that button - and go with the power of Porting instead!".localize().format(Tutorial.getClickVerb()), {
                    type: NotificationType.AutoPopup
                }))
            }
        }
    }
    Research.SpecialItems.push(Research.GamePorts)

    // Tutorials Allowed Determinance
    const AreTutorialsAllowed = function() {
        return GameFlags.TUTORIAL_DISABLED ? false : (SettingsGameplay.isTutorialOff() || !SettingsGameplay.alwaysShowTutorials())
    }

    const CanShowTutorial = function(tutorial) {
        return AreTutorialsAllowed() && (DataStore.getTutorialSettings()["tutorials-shown"].indexOf(tutorial) == -1 || GameManager.company.tutorialMessagesShown.indexOf(tutorial) == -1)
    }

    const TutorialShown = function(tutorial) {
        if(CanShowTutorial(tutorial)) {
            const keys = DataStore.getTutorialSettings()

            GameManager.company.tutorialMessagesShown.push(tutorial)

            keys["tutorials-shown"].push(tutorial)
            DataStore.saveSettings()
        }
    }

    /**
     * There's probably a better way to handle adding this to the context menu but this is an incredibly
     * efficient hack and is easy without having to shift through the entire API - so who cares!
    **/

    // Handle adding Game Porting to Context Menu
    const OriginalShowContextMenu = UI.showContextMenu

    UI.showContextMenu = function(items, vector2) {
        const Company = GameManager.company
        const SelectedWorker = 1 < Company.currentLevel ? UI.getCharUnderCursor() : Company.staff[0]
        
        if(!SelectedWorker) {
            // Determine if we're able to develop a game and if so work out other details
            if(GameManager.isIdle() && !GameManager.currentEngineDev && 0 < Company.staff.filter(function(worker) {
                return worker.state === CharacterState.Idle
            }).length) {
                // Determine if we even have our desired research goal
                if((GameManager.company.gameLog.length > 0) && 0 <= Company.researchCompleted.indexOf(Research.GamePorts)) {
                    items.push({
                        label: "Port Game...".localize(),
                        icon: OurDirectory + "/contextIcon.svg",
                        action: function() {
                            Sound.click()

                            /* Process for porting
                                1. Have user select game
                                    => UI.showGameHistory(callback)
                                        - Callback is for when something is selected
                                        - GameManager.flags.selectedGameId, GameManager.company.currentGame, GameManager.company.getGameById(a)
                                        - GameManager.flags.selectedGameId = null
                                    - Continue to step 2. if their is a game chosen
                                2. Create new game
                                    => GameManager.company.createNewGame()
                                        - The idea is that we create a game so we can select the platform
                                3. Create game creation instance
                                    => UI.showGameDefinition(GameManager.company, {close: true, disableCheckForNotification: true}) 
                                        - Used so that platform picking does not error
                                4. Display platforms and have user select one
                                    => UI.pickPlatformClick($(document.createElement("div")))
                                        - Use empty div so it can guarantee success
                                    - If a platform is selected then move to step 5. Otherwise quit.
                                5. Set current in-development game properties to one we are porting + new platform
                                6. Entre research stage (development time for port)
                                    - Code for this is actually borrowed from the source, I just modified some of it
                                    so that it doesn't actually create a new engine.
                                7. Launch Game 
                            */

                            // Ask user to select a game
                            GameManager.flags.selectGameActive = true

                            UI.showGameHistory(function() {
                                const SelectedGame = GameManager.flags.selectedGameId

                                if(SelectedGame) {
                                    const Game = Company.getGameById(SelectedGame)

                                    // Create a new game
                                    const NewGame = Company.createNewGame()

                                    // Prompt user with default-game-creation-prompt
                                    UI.showGameDefinition(Company, {close: true, disableCheckForNotification: true})
                                
                                    // Prompt user to select a platform
                                    UI.pickPlatformClick($(document.createElement("div")))

                                    // Temporary watcher for platform selection
                                    const OriginalSelectPlatform = UI._selectPlatform

                                    UI._selectPlatform = function(selectedPlatform, element) {
                                        const LicencedPlatform = Company.licencedPlatforms.first(function(platform) {
                                            return selectedPlatform === platform.id
                                        })

                                        // If we have it and than continue, otherwise proceed with normal routine
                                        if(LicencedPlatform) {
                                            // Close dialogs
                                            UI.closeAllDialogs()

                                            // Reset
                                            UI._selectPlatform = OriginalSelectPlatform

                                            // Determine name for expenses and for prompts
                                            const TitleName = "{0} {1} Port".format(Game.title, LicencedPlatform.name)

                                            // Call to prompt tutorial if needed
                                            console.log(CanShowTutorial("CreatingPorts"), AreTutorialsAllowed())
                                            if(CanShowTutorial("CreatingPorts")) {
                                                TutorialShown("CreatingPorts")
                                            
                                                Company.notifications.push(new Notification("Developing Ports".localize("heading"), "Ports take time to complete - we all know that. But what you may not have known is that there's no changing values and sliders and that stuff, you already have that done! But, there's a catch: The more expensive your port is, the longer it takes to make. That won't be a problem if you have a team working on it, more members = faster ports!".localize(), {
                                                    type: NotificationType.AutoPopup
                                                }))
                                            }

                                            // Determine cost for port
                                            const Cost = Game.costs * 1.5 + LicencedPlatform.developmentCosts

                                            // Transfer & Set Settings from our selected game to our new game
                                            NewGame.bugs = Math.floor(Game.bugs / 2)
                                            NewGame.costs = Cost
                                            NewGame.designPoints = Game.designPoints * 1.25
                                            NewGame.featureLog = $.extend(true, [], Game.featureLog)
                                            NewGame.features = $.extend(true, [], Game.features)
                                            NewGame.flags = $.extend(true, {}, Game.flags)
                                            NewGame.gameSize = Game.gameSize
                                            NewGame.genre = Game.genre
                                            NewGame.hypePoints = Game.hypePoints * 1.5
                                            NewGame.missionLog = $.extend(true, [], Game.missionLog)
                                            NewGame.platforms = [LicencedPlatform]
                                            NewGame.releaseWeek = Company.currentWeek
                                            NewGame.state = 4
                                            NewGame.targetAudience = Game.targetAudience
                                            NewGame.topic = Game.topic
                                            NewGame.unitPrice = Game.unitPrice * 1.25
                                            NewGame.title = Game.title
                                            NewGame.researchFactor = Game.researchFactor
                                            NewGame.technologyPoints = Game.technologyPoints
                                            NewGame.freeBugCount = Math.floor(Game.freeBugCount * 2)
                                            NewGame.portOf = Game

                                            // Charge company for port
                                            Company.adjustCash(-Cost, TitleName)

                                            // Determine point cost for port
                                            const PointCost = Math.floor(Math.min((1680.933 + (75.13457 - 1680.933) / Math.pow(1 + (Cost / 8716210), 0.6768567)) / 0.75, 1000))

                                            // Filler parts to make it work
                                            const Parts = [
                                                {
                                                    canResearch: function() {
                                                        return false;
                                                    },
                                                    category: "Story/Quests",
                                                    categoryDisplayName: "Story/Quests",
                                                    Experience: 0,
                                                    group: "story",
                                                    id: "Linear Story",
                                                    name: "Porting...",
                                                    v: 2
                                                }
                                            ]

                                            // Set current enginge development tag
                                            GameManager.currentEngineDev = {
                                                name: TitleName,
                                                progress: 0,
                                                cost: Cost,
                                                technologyPoints: PointCost,
                                                remainingPoints: PointCost,
                                                remainingPointsDisplay: PointCost,
                                                parts: Parts,
                                                currentPart: Parts[0]
                                            }

                                            // Tell visual manager to display engine UI
                                            VisualsManager.startCreateEngine()

                                            // Manually update
                                            const GameStatusBar = VisualsManager.gameStatusBar
                                            const ManualUpdateInterval =  setInterval(function() {
                                                VisualsManager.gameStatusBar.enginePoints.updatePoints(GameManager.currentEngineDev.remainingPointsDisplay)
                                            }, 50)

                                            // Set title
                                            GameStatusBar.gameDetailText.text = ("Porting " + Game.title).localize()

                                            // Filter staff to adjust who we animate
                                            const StaffToAnimate = Company.staff.filter(function(worker) {
                                                return worker.state === CharacterState.Idle
                                            })

                                            // Tell workers to - well, start working!
                                            StaffToAnimate.forEach(function(worker) {
                                                worker.startWorking()
                                            })

                                            // Resume game
                                            GameManager.resume(true)

                                            // Overwrite engine finishing so we don't actually create a new engine
                                            const OriginalFinishEngine = GameManager.finishEngine
                                            var processed

                                            GameManager.finishEngine = function() {
                                                if (processed || 1 > GameManager.currentEngineDev.progress) {
                                                    return
                                                }           

                                                // State that we've already gone through this
                                                processed = true

                                                // Stop updating
                                                clearInterval(ManualUpdateInterval)

                                                // Remove current engine development tag
                                                GameManager.currentEngineDev = null
                                                
                                                // Filter staff to adjust who we animate
                                                const StaffToAnimate = Company.staff.filter(function(worker) {
                                                    return worker.state === CharacterState.CreateEngine
                                                })

                                                // Tell workers to - well, stop working!
                                                StaffToAnimate.forEach(function(worker) {
                                                    worker.state = CharacterState.Idle
                                                })

                                                // Show notification
                                                Company.notifications.push(new Notification("Port complete!".localize("heading"), "Your new Port of '{0}' is now complete!".localize().format(Game.title), {
                                                    type: NotificationType.AutoPopup
                                                }))

                                                // Display tutorial
                                                if(CanShowTutorial("ReleasingPorts")) {
                                                    TutorialShown("ReleasingPorts")
                                                
                                                    Company.notifications.push(new Notification("Releasing Ports".localize("heading"), "Congratulations on your first ported-game! Porting is super easy (as you've seen) and takes a bit of time based on the price; but - again - there's another hidden secret. Ported games cost more to buy, so all that money that you put into porting your game: You are sure to get it right back if your game works on other consoles! Oh... did I mention that some games don't do too well on other consoles.. {n} Porting also helps you take down bugs along the way, so if you released your original game with bugs - well say goodbye to most of them because you've weeded them out. Ports are awesome - but they take some time to get used to - so port to your hearts content and watch the earnings roll in!".localize(), {
                                                        type: NotificationType.AutoPopup
                                                    }))
                                                }

                                                // Finish engine visually
                                                VisualsManager.finishEngine()

                                                // Release game
                                                General.releaseGame(Company)

                                                // Autosave
                                                GameManager.autoSave()

                                                // Restore
                                                GameManager.finishEngine = function() {
                                                    if(GameManager.currentEngineDev) {
                                                        OriginalFinishEngine()
                                                    }
                                                }
                                            }
                                        } else {
                                            OriginalSelectPlatform(selectedPlatform, element)
                                        }
                                    }

                                    $('.closeDialogButton')[0].addEventListener('click', function() {
                                        UI.closeAllDialogs()

                                        UI._selectPlatform = OriginalSelectPlatform

                                        Company.currentGame = null

                                        GameManager.resume()
                                    })
                                }

                                delete GameManager.flags.selectGameActive
                                delete GameManager.flags.selectedGameId
                            })

                            GameManager.flags.selectGameActive = false;
                            GameManager.flags.createPack = false;
                        }
                    })
                }
            }
        }

        OriginalShowContextMenu(items, vector2)
    }
}